from typing import TYPE_CHECKING
from si_prefix import si_format

if TYPE_CHECKING:
    from .container import Container


class Audio:
    """An audio stream"""

    def __init__(self, container: 'Container', data: dict):
        self.container = container
        self.data = data

    @property
    def index(self) -> int:
        """The index of the stream"""
        return self.data['index']

    @property
    def codec(self) -> str:
        """The codec of the stream"""
        return self.data.get('codec_name', '--Unknown--')

    @property
    def channel_layout(self) -> str:
        """The channel layout of the stream"""
        return self.data.get('channel_layout', '?')

    @property
    def bit_rate(self) -> str:
        """The bitrate of the stream si formatted or N/A if not available"""
        return si_format(int(self.data['bit_rate'])) + 'b/s' if self.data.get('bit_rate') else 'N/A'

    @property
    def language(self) -> str:
        """The language of the stream or 'und'"""
        return self.data.get('tags', {}).get('language', 'und')

    @property
    def is_forced(self) -> bool:
        """Whether this is a forced stream"""
        return self.data.get('disposition', {}).get('forced', False)

    @property
    def disposition(self) -> str:
        """The properties of the stream as comma seperated list as a string"""
        return ','.join(k for k, v in self.data['disposition'].items() if v)

    @property
    def title(self) -> str:
        """The title of the stream"""
        return self.data.get('tags', {}).get('title', '')

    def to_dict(self) -> dict:
        return {
            'index': self.index,
            'codec': self.codec,
            'channel_layout': self.channel_layout,
            'bit_rate': self.bit_rate,
            'language': self.language,
            'is_forced': self.is_forced,
            'disposition': self.disposition,
            'title': self.title,
        }
