import json
import os
from pathlib import Path
from subprocess import check_output, CalledProcessError
from typing import List, Optional

from rizzle_lib import ProgressPrinter

from .audio import Audio
from .exception import NoVideoException
from .format import Format
from .helper import croak
from .subtitle import Subtitle
from .video import Video


class Container:
    """A container, consisting of multiple streams"""

    def __init__(self, file: Path, ffprobe_json: dict, mtime: float):
        self.file = file
        self.data = ffprobe_json
        self.dirname = os.path.dirname(self.file.parent)
        self.format = Format(self, self.data['format'])
        self.video: Optional[Video] = None
        self.chapters = len(self.data['chapters'])
        self.audios: List[Audio] = []
        self.subtitles: List[Subtitle] = []
        self.mtime = mtime
        for s in self.data['streams']:
            if s['codec_type'] == 'video':
                if not self.video:
                    self.video = Video(self, s)
            elif s['codec_type'] == 'audio':
                self.audios.append(Audio(self, s))
            elif s['codec_type'] == 'subtitle':
                self.subtitles.append(Subtitle(self, s))
            elif s['codec_type'] == 'data':
                pass
            elif s['codec_type'] == 'attachment':
                pass
            else:
                croak(f"Unknown codec type: {s['codec_type']}")
        if not self.video:
            raise NoVideoException(f"No video stream found in {self.file}")

    @classmethod
    def parse_from_file(cls, file: Path) -> "Container":
        """Parse a single file via ffprobe and return it or raise a NoVideoException"""
        try:
            data = json.loads(check_output([
                'ffprobe', '-i', str(file),
                '-show_streams', '-show_entries', 'format:stream:chapter',
                '-v', 'quiet', '-of', 'json',
            ]))
            return cls(file, data, file.stat().st_mtime)
        except CalledProcessError as e:
            raise NoVideoException from e

    @classmethod
    def parse_from_files(cls, files: List[Path], is_recursive: Optional[bool] = None) -> List["Container"]:
        """Parse multiple files via ffprobe and return the list of valid ones"""
        pp = ProgressPrinter()
        pp.update_var.put(['max', '=', len(files)])
        pp.start()
        containers = []

        def add_file(p: Path):
            try:
                containers.append(cls.parse_from_file(p))
            except NoVideoException:
                pass

        for f in sorted(files):
            if f.is_dir() and is_recursive:
                for s in sorted(f.rglob('*')):
                    if s.is_file():
                        add_file(s)
            elif f.is_file():
                add_file(f)
            pp.update_var.put(['current', '+', 1])

        pp.end.set()
        pp.join()

        return containers

    def to_json(self) -> str:
        return json.dumps({
            'file': str(self.file),
            'data': self.data,
            'mtime': self.mtime,
        })

    @classmethod
    def from_json(cls, json_str) -> 'Container':
        j = json.loads(json_str)
        return cls(Path(j['file']), j['data'], j['mtime'])

    def to_dict(self) -> dict:
        return {
            'filename': self.file.name,
            'format': self.format.to_dict(),
            'video': self.video.to_dict(),
            'chapters': self.chapters,
            'audios': [a.to_dict() for a in self.audios],
            'subtitles': [s.to_dict() for s in self.subtitles],
            'mtime': self.mtime,
        }
