from typing import TYPE_CHECKING
from si_prefix import si_format, si_parse

if TYPE_CHECKING:
    from .container import Container


class Format:
    def __init__(self, container: 'Container', data: dict):
        self.container = container
        self.data = data

    @property
    def bitrate(self) -> str:
        """The bitrate as si formatted string"""
        return si_format(int(self.data['bit_rate'])) + 'b/s' if self.data.get('bit_rate') else 'N/A'

    @property
    def bitrate_numeric(self) -> float:
        """The bitrate as a float value, or 9e15 if not valid"""
        if self.bitrate == 'N/A':
            return 9e15
        return si_parse(self.bitrate.split('b')[0])

    @property
    def duration(self) -> str:
        """The duration as a string in minutes"""
        dur = self.data.get('duration', 'N/A')
        if dur == 'N/A':
            return dur
        return '{} min.'.format(int(float(dur) / 60))

    @property
    def duration_s(self) -> int:
        """The duration as an int in seconds, or -1 if not available"""
        dur = self.data.get('duration', '-1')
        return int(float(dur))
    
    @property
    def duration_s_fractal(self) -> float:
        """The duration as a float in seconds, or -1 if not available"""
        dur = self.data.get('duration', '-1')
        return float(dur)

    @property
    def size(self) -> str:
        """The size as a si formatted string"""
        return '{}B'.format(si_format(int(self.data['size']))) if 'size' in self.data else 'N/A'

    @property
    def size_b(self) -> int:
        """The size in bytes"""
        return int(self.data.get('size'))

    def to_dict(self) -> dict:
        return {
            'bitrate': self.bitrate,
            'bitrate_numeric': self.bitrate_numeric,
            'duration': self.duration,
            'duration_s': self.duration_s,
            'duration_s_fractal': self.duration_s_fractal,
            'size': self.size,
            'size_b': self.size_b,
        }
